private import mygl;

private import common;
private import charactor;
private import objspecs;
private import game;
private import myglbf;
private import texture;

abstract class Effect : Charactor {
    this(float x, float y) {
        super();
        x_ = x;
        y_ = y;
    }
}

// two purpose class...
class StringEffect : Effect {
    mixin HasRelativePush;

public:
    this(float x, float y, char[] str, float w, float h,
         float r, float g, float b) {
        this(x, y, str, w, h, r, g, b, 100);
    }

    this(float x, float y, char[] str, float w, float h,
         float r, float g, float b, int lifetime, int turn) {
        this(x, y, str, w, h, r, g, b, lifetime);
        turn_ = turn;
    }

    this(float x, float y, char[] str, float w, float h,
         float r, float g, float b, int lifetime) {
        super(x, y);
        str_ = str;
        w_ = w;
        h_ = h;
        r_ = r;
        g_ = g;
        b_ = b;
        lifetime_ = lifetime;
        absolute_ = lifetime_ == 0;

        for (int i = 0; i < NUM; i++) {
            float a = rnd(2*PI);
            osx_[i] = sin(a);
            osy_[i] = cos(a);
            ox_[i] = DIM_TURN*osx_[i];
            oy_[i] = DIM_TURN*osy_[i];
        }
    }

    override bit alive() {
        return !lifetime_ || turn_ < lifetime_+DIM_TURN;
    }

    override void move() {
        super.move();

        if (turn_ > 0 && (turn_ < DIM_TURN ||
                          (lifetime_ && turn_ > lifetime_)))
        {
            for (int i = 0; i < NUM; i++) {
                ox_[i] -= osx_[i];
                oy_[i] -= osy_[i];
            }
        }
    }

    override void draw() {
        if (turn_ < 0) return;

        if (!absolute_) {
            pushMatrix();

            for (int i = 0; i < NUM; i++) {
                float a = 0.4;
                drawString(str_, ox_[i], oy_[i], w_, h_, r_, g_, b_, a);
            }

            glPopMatrix();
        }
        else {
            for (int i = 0; i < NUM; i++) {
                float a = 0.4;
                drawString(str_, x_+ox_[i], y_+oy_[i],
                           w_, h_, r_, g_, b_, a);
            }
        }
    }

    StringEffect right() {
        x_ -= str_.length*12*w_/14;
        return this;
    }

    StringEffect center() {
        x_ -= str_.length*6*w_/14;
        return this;
    }

    StringEffect speed(float sx, float sy) {
        sx_ = sx;
        sy_ = sy;
        return this;
    }

    StringEffect absolute() {
        absolute_ = true;
        return this;
    }

private:
    char[] str_;
    float w_, h_;
    float r_, g_, b_;
    int lifetime_;

    static const int NUM = 3;
    static const int DIM_TURN = 30;
    float[3] ox_, oy_;
    float[3] osx_, osy_;

    bool absolute_;

}

class ExpEffect : Effect {
    mixin HasRelativePush;

public:
    this(float x, float y) {
        super(x, y);
        angle_ = rnd(360);
        size_ = rnd(1.0)+1.0;
        turn_ = 10;
    }

    this(float x, float y, int t) {
        this(x, y);
        turn_ = t;
        size_ = rnd(1.0)+1.5;
    }

    override bit alive() {
        return turn_ < 60;
    }

    override void draw() {
        if (turn_ < 0) return;

        pushMatrix();

        glRotatef(angle_, 0, 0, 1);

        float[4] texcoord = Texture.prepare(TextureID.TEX_BARRIER);

        glColor4f(1, 1, 1, 0.7-turn_*0.011);

        glBegin(GL_QUADS);
        glTexCoord2f(texcoord[0], texcoord[1]);
        glVertex2f(-turn_*size_*2, -turn_*size_);
        glTexCoord2f(texcoord[0], texcoord[3]);
        glVertex2f(-turn_*size_*2, turn*size_);
        glTexCoord2f(texcoord[2], texcoord[3]);
        glVertex2f(turn_*size_*2, turn_*size_);
        glTexCoord2f(texcoord[2], texcoord[1]);
        glVertex2f(turn_*size_*2, -turn_*size_);
        glEnd();

        glPopMatrix();
    }

private:
    float angle_;
    float size_;
}
