private import charactor;
private import game;
private import shot;
private import common;
private import objspecs;
private import mygl;
private import myglbf;
private import texture;
private import player;
private import enemy_view;
private import enemy_move;
private import enemy_mgr;

private import pcl;

private import std.math;

extern (C) void enemy_spawn(void* p) {
    Enemy e = cast(Enemy)p;
    while (true) {
        e.moveTask();
    }
}

class Enemy : Charactor {
    mixin HasLife;
    mixin HasRelativePush;
    mixin HasClipping;

    this(float x, float y, EnemyView v, EnemyMove m, float param, int pat) {
        x_ = x;
        y_ = y;

        param_ = param;
        pattern_ = pat;

        view_ = v;
        move_ = m;

        sx_ = 0;
        sy_ = 0;

        setLife(1.0);

        size_ = 20;

        version (MacOSX) {
            static const int CO_STACK_SIZE = (8 * 1024);
        }
        else { 
            static const int CO_STACK_SIZE = (16 * 1024);
       }
        coro_ = co_create(&enemy_spawn, this, null, CO_STACK_SIZE);

        move_.init(this);
    }

    this(float x, float y, float param, int pat) {
        this(x, y, new ProtonView(x, y), EnemyMgr.getRandomMove(), param, pat);
    }

    ~this() {
        co_delete(coro_);
    }

    override void move() {
        if (barrierCnt_ > 0) barrierCnt_--;
        if (wait_ > 0) {
            wait_--;
        }
        else {
            co_call(coro_);
//            move_.move(this);
        }

        super.move();

        view_.setXY(x_, y_);
        view_.move();

        clipPos();
    }

    void moveTask() {
        move_.move(this);
    }

    override void draw() {
        if (!Game.obj.isInside(x_, y_, 10)) return;

        view_.draw();

        pushMatrix();

        int ilife = cast(int)life_;
        char[] lifeStr = tostr(ilife) ~ `.` ~ tostr((ilife == life_) ? 0 : 5);
        drawString(lifeStr, 10, 20, 6, 12, 1,1,1, 0.9);

        if (longWait_ != 0) {
            int size = 33;
            if (barrierCnt_ < 20) {
                size = 13 + barrierCnt_;
            }
            else if (longWait_-barrierCnt_ < 20) {
                size = 13 + longWait_ - barrierCnt_;
            }

            glColor4f(1, 1, 1, size * 0.01);

            float[] texcoord = Texture.prepare(TextureID.TEX_BARRIER);

            float tx = size;
            float ty = size;
            float rs() { return 0; }
            glBegin(GL_QUADS);
            glTexCoord2f(texcoord[0], texcoord[1]);
            glVertex2f(-tx-rs(), -ty-rs());
            glTexCoord2f(texcoord[0], texcoord[3]);
            glVertex2f(-tx-rs(), ty+rs());
            glTexCoord2f(texcoord[2], texcoord[3]);
            glVertex2f(tx+rs(), ty+rs());
            glTexCoord2f(texcoord[2], texcoord[1]);
            glVertex2f(tx+rs(), -ty-rs());
            glEnd();
        }

        glPopMatrix();
    }

    void setLife(float life) {
        life_ = life;
        score_ = cast(int)(life * 10000);
        if (life_ == 0.75) life_ = 1.0;
    }

    int score() { return score_; }

    void damage(float damage, float damageAngle) {
        if (Game.obj.mode != Game.Mode.EASY &&
            (life_ < 2.1 &&
             (longWait_ != 0 &&
              longWait_-barrierCnt_ > 20 &&
              barrierCnt_ > 20)))
        {
            life_ -= 0.5 * damage;
        }
        else {
            life_ -= damage;
        }

        if (alive) {
            x_ += sin(damageAngle) * 200;
            y_ += cos(damageAngle) * 200;
            clipPos();
            view_.setXY(x_, y_);
            view_.move();
        }
        else {
            view_.destroy(damageAngle);

            Game.obj.delEnemy(pattern_);
        }
    }

private:
    EnemyView view_;
    EnemyMove move_;

    int wait_;
    int longWait_;
    int barrierCnt_;

    coroutine_t coro_;
    float param_;

    int pattern_;

    int score_;

public:
    // --- for Io ---

    void shotAbs(float s, float a) {
        shotAbsXY(s*sin(a), s*cos(a));
    }

    void shotAbsXY(float sx, float sy) {
        float[3] col;
        if (score_ < 20000) {
            col[0] = 0.7;
            col[1] = 1;
            col[2] = 0.9;
        }
        else {
            col[0] = 0.7;
            col[1] = 0.9;
            col[2] = 1.0;
        }
        Game.obj.addShot(x_, y_, sx, sy, col);
    }

    void sleep(int t) {
        wait_ = t;
        if (t > 0) {
            if (t > 60) {
                if (barrierCnt_ <= 0) {
                    setBarrier(t);
                }
            }
            while (wait_ > 0) {
                co_resume();
            }
        }
    }

    void setBarrier(int t) {
        barrierCnt_ = longWait_ = t;
    }

    float getPlayerAngle() {
        Player p = Game.obj.player;
        return .angle(x_, y_, p.x, p.y);
    }

    float getPlayerDistance() {
        Player p = Game.obj.player;
        return .distance(x_, y_, p.x, p.y);
    }

    float getPlayerDistance2() {
        Player p = Game.obj.player;
        return .distance2(x_, y_, p.x, p.y);
    }

    void setSpeed(float s, float a) {
        sx_ = s * sin(a);
        sy_ = s * cos(a);
    }

    void setSpeedXY(float sx, float sy) {
        sx_ = sx;
        sy_ = sy;
    }

    float angle() {
        return .angle(sx_, sy_);
    }

    float param() { return param_; }

}

