private import enemy;
private import common;
private import game;

abstract class EnemyMove {
    this() {
        turn = 0;
    }

    void init(Enemy e) {}

    void move(Enemy e) {
        turn++;
        move_(e);
    }

protected:
    int turn;

    abstract void move_(Enemy e);

    // workaround for cyclic define...
    //Game.Mode mode() {
    int mode() {
        return Game.obj.mode;
    }
}

class GrindShotMove : EnemyMove {
    override void move_(Enemy e) {
        for (int i = 0; i < 40; i++) {
            for (int j = 0; j < cast(int)e.param; j++) {
                e.shotAbs(rrnd(2.0)+1, rrnd(PI*2));
            }
            e.shotAbs(rrnd(2.0)+1, e.getPlayerAngle());
            e.sleep(3);
        }
        for (int i = 0; i < 20; i++) {
            for (int j = 0; j < cast(int)e.param; j++) {
                e.shotAbs(rrnd(2.0)+1, rrnd(PI*2));
            }
        }
        e.sleep(120);
    }
}

class SurroMove : EnemyMove {
    override void move_(Enemy e) {
        e.setSpeed(2, e.getPlayerAngle() + angle*PI);
        for (int i = 0; i < 5; i++) {
            e.shotAbs(4, e.getPlayerAngle()+(i-2)*0.03*PI);
        }
        e.sleep(80);
    }
    this() {
        angle = (rrnd(2) == 0) ? 0.3 : -0.3;
    }
    float angle;
}

class AlleMove : EnemyMove {
    override void move_(Enemy e) {
        for (int i = 0; i < 30; i++) {
            e.shotAbs(rrnd(2.0)+1, rrnd(2*PI));
            e.sleep(1);
        }
        e.setSpeed(1, rrnd(2*PI));
        e.sleep(100);
    }
}

class AtackMove : EnemyMove {
    override void move_(Enemy e) {
        e.setSpeed(2, e.getPlayerAngle()+rrnd(2.0)-1.0);
        for (int i = 0; i < 10; i++) {
            e.shotAbs(2.3, e.getPlayerAngle()+rrnd(0.6)-0.3);
            e.sleep(5);
        }
        e.sleep(70);
    }
}

class RollFireMove : EnemyMove {
    override void move_(Enemy e) {
        for (int i = 0; i < 100; i++) {
            if (mode != Game.Mode.EASY || (i & 1))
                e.shotAbs(1.5, PI/50*i);
            e.sleep(2);
        }
    }
}

class TrackMove : EnemyMove {
    override void init(Enemy e) {
        e.setSpeed(0.5, e.getPlayerAngle());
    }

    override void move_(Enemy e) {
        for (int i = 0; i < 5; i++) {
            for (int j = 0; j < 5; j++) {
                float angle = e.angle;
                if (relAngle(angle - e.getPlayerAngle) < 0) {
                    angle += 0.05;
                }
                else {
                    angle -= 0.05;
                }
                e.setSpeed(0.5, angle);
                e.sleep(4);
            }
            e.shotAbs(1.7, e.getPlayerAngle());
        }
        e.sleep(100);
    }
}

class TwoWayMove : EnemyMove {
    override void move_(Enemy e) {
        for (int i = 0; i < 4; i++) {
            float r = 0.1 + i*0.02;
            e.shotAbs(1.7, e.getPlayerAngle()+r);
            e.shotAbs(1.7, e.getPlayerAngle()-r);
            e.sleep(10);
        }
        e.sleep(100);
    }
}

class ThreeWayMove : EnemyMove {
    override void init(Enemy e) {
        angle_ = e.param;
    }

    override void move_(Enemy e) {
        for (int i = 0; i < 4; i++) {
            for (int j = 0; j < 10; j++) {
                 e.setSpeed(1.7, angle_);
                 angle_ += 0.05;
                 e.sleep(1);
            }

            float r = 0.2 + i*0.02;
            e.shotAbs(1.7, e.getPlayerAngle()+r);
            e.shotAbs(1.7, e.getPlayerAngle());
            e.shotAbs(1.7, e.getPlayerAngle()-r);
        }
        e.setBarrier(100);
        for (int i = 0; i < 50; i++) {
            e.setSpeed(1.7, angle_);
            angle_ += 0.05;
            e.sleep(1);
        }
    }

private:
    float angle_;

public: // temporary added for gdc

}

class SniperMove : EnemyMove {
    override void move_(Enemy e) {
        e.setBarrier(140);
        e.sleep(20);
        float a = (3-e.param) * PI;
        e.setSpeed(4, e.getPlayerAngle()+rrnd(a)-a*0.5);
        e.sleep(100);
        e.setSpeed(0, 0);
        e.sleep(20);

        for (int i = 0; i < cast(int)(30*e.param); i++) {
            float a = 0.6 * e.param;
            if (mode != Game.Mode.EASY || (i & 1))
                e.shotAbs(2.5, e.getPlayerAngle()+rrnd(a)-a*0.5);
            e.sleep(1);
        }
    }
}

class ThreeWayMoveNext : EnemyMove {
    override void init(Enemy e) {
        angle_ = e.param;
    }

    override void move_(Enemy e) {
        for (int i = 0; i < 10; i++) {
            for (int j = 0; j < 5; j++) {
                 e.setSpeed(1.7, angle_);
                 angle_ += 0.05;
                 e.sleep(1);
            }

            float r = 0.2 + i*0.02 + rrnd(0.2);
            e.shotAbs(2.1, e.getPlayerAngle()+r);
            e.shotAbs(2.1, e.getPlayerAngle());
            e.shotAbs(2.1, e.getPlayerAngle()-r);
        }
        e.setBarrier(100);
        for (int i = 0; i < 50; i++) {
            e.setSpeed(1.7, angle_);
            angle_ += 0.05;
            e.sleep(1);
        }
    }

private:
    float angle_;

public: // temporary added for gdc

}

class FastSurroMove : EnemyMove {
    override void move_(Enemy e) {
        float d = e.getPlayerDistance2();
        float a = PI*0.3;
        if (e.x < 100 || e.x > Game.obj.fieldWidth-100 ||
            e.y < 100 || e.y > Game.obj.fieldHeight-100) {
            a = PI*0.1;
        }
        else if (d < 40000) a += PI * 0.2;
        else if (d < 100000) a += PI * 0.1;
        e.setSpeed(2.5+e.param, e.getPlayerAngle() - a);
        e.shotAbs(2, e.getPlayerAngle());
        e.sleep(15);
    }
}

class FanMove : EnemyMove {
    override void move_(Enemy e) {
        float a = rrnd(2*PI);
        e.setSpeed(0.5+rrnd(1.0), rrnd(2*PI));
        for (int i = 0; i < 110; i++) {
            e.shotAbs(2.1, a);
            a += PI*0.703;
            e.sleep(1);
        }
        e.setSpeed(0, 0);
        e.sleep(120);
    }
}

class GaraMove : EnemyMove {
    override void move_(Enemy e) {
        e.setBarrier(140);
        e.sleep(20);
        float a = PI;
        e.setSpeed(4, e.getPlayerAngle()+rrnd(a)-a*0.5);
        e.sleep(100);
        e.setSpeed(0, 0);
        e.sleep(20);

        // use about 100 frames

        int r = rrnd(2);

        int six = (mode != Game.Mode.EASY) ? 6 : 4;

        if (r == 0) {
            int pm = (rrnd(2) == 0) ? 1 : -1;
            float a = 0.6 * pm;
            for (int i = 0; i < 10; i++) {
                for (int j = 0; j < six; j++) {
                    e.shotAbs(1.5+0.5*j, e.getPlayerAngle()+a);
                }
                a -= 0.12 * pm;
                e.sleep(5);
            }
        }
        else if (r == 1) {
            for (int i = 0; i < 10; i++) {
                for (int j = 0; j < six; j++) {
                    e.shotAbs(1.5+0.5*j, e.getPlayerAngle());
                }
                e.sleep(5);
            }
        }
    }
}

class AMove : EnemyMove {
    override void move_(Enemy e) {
        e.setBarrier(80);
        e.sleep(10);
        float a = PI*1.2;
        e.setSpeed(4, e.getPlayerAngle()+rrnd(a)-a*0.5);
        e.sleep(60);
        e.setSpeed(0, 0);
        e.sleep(10);

        for (int i = 0; i < 30; i++) {
            float a = 1.8;
            a = e.getPlayerAngle()+rrnd(a)-a*0.5;
            e.shotAbs(rrnd(1.0)+2, a);
            if (mode != Game.Mode.EASY || (i & 1))
                e.shotAbs(rrnd(1.0)+3, a);
            e.sleep(1);
        }
    }
}

class BMove : EnemyMove {
    override void move_(Enemy e) {
        int period = 120;
        for (int i = 0; i < period; i++) {
            float x = 5.2 * cos(2.0*PI*i/period);
            float y = 0.5;
            float a = e.getPlayerAngle();
            e.setSpeedXY(cos(a)*x+sin(a)*y, -sin(a)*x+cos(a)*y);

            if (mode != Game.Mode.EASY || (i & 3) == 0)
                e.shotAbs(1, rrnd(2*PI));

            e.sleep(1);
        }
    }
}

class CMove : EnemyMove {
    override void init(Enemy e) {
        angle_ = e.param;
    }

    override void move_(Enemy e) {
        int tt = (mode != Game.Mode.EASY) ? 13 : 10;

        for (int i = 0; i < tt; i++) {
            for (int j = 0; j < 3; j++) {
                 e.setSpeed(1.7, angle_);
                 angle_ += 0.05;
                 e.sleep(1);
            }

            float r = 0.7 - i*0.03;
            e.shotAbs(1.5+i*0.3, e.getPlayerAngle()+r);
            e.shotAbs(1.5+i*0.3, e.getPlayerAngle());
            e.shotAbs(1.5+i*0.3, e.getPlayerAngle()-r);
        }
        e.setBarrier(70);
        for (int i = 0; i < 35; i++) {
            e.setSpeed(1.7, angle_);
            angle_ += 0.05;
            e.sleep(1);
        }
    }

private:
    float angle_;

public: // temporary added for gdc

}

class DMove : EnemyMove {
    override void move_(Enemy e) {
        float a = rrnd(2*PI);
        e.setSpeed(0.1, rrnd(2*PI));
        for (int i = 0; i < 110; i++) {
            e.shotAbs(2.1, a);
            if (mode != Game.Mode.EASY || (i & 1))
                e.shotAbs(2.1, a+PI);
            a += PI*0.2*sin(i*i*0.001);
            e.sleep(1);
        }
        e.setSpeed(0, 0);
        e.sleep(70);
    }
}

