private import drawobjs;
private import game;
private import mygl;
private import enemy;
private import player;
private import common;
private import myglbf;
private import texture;

class PsynoDraw : DrawObj {
    override void draw() {
        float[4] texcoord = Texture.prepare(TextureID.TEX_PSYNO);
        glColor4f(1, 1, 1, 0.3);
        glBegin(GL_QUADS);
        glTexCoord2f(texcoord[0], texcoord[1]);
        glVertex2f(480, 0);
        glTexCoord2f(texcoord[2], texcoord[1]);
        glVertex2f(480, 480);
        glTexCoord2f(texcoord[2], texcoord[3]);
        glVertex2f(640, 480);
        glTexCoord2f(texcoord[0], texcoord[3]);
        glVertex2f(640, 0);
        glEnd();

        glDisable(GL_TEXTURE_2D);
    }
}

class MapDraw : DrawObj {
    override void draw() {
        int offx = 488;
        int offy = 8;

        glLineWidth(1);

        glColor4f(1, 1, 1, 0.5);

        glBegin(GL_LINES);
        glVertex2f(offx, offy);
        glVertex2f(offx+144, offy);
        glVertex2f(offx+144, offy);
        glVertex2f(offx+144, offy+144);
        glVertex2f(offx+144, offy+144);
        glVertex2f(offx, offy+144);
        glVertex2f(offx, offy+144);
        glVertex2f(offx, offy);
        glEnd();

        glBegin(GL_QUADS);

        glColor3f(0, 0, 1);
        foreach (Enemy e; Game.obj.enemies()) {
            float x = e.x * 144 / Game.obj.fieldWidth + offx;
            float y = e.y * 144 / Game.obj.fieldHeight + offy;
            glVertex2f(x+1, y+1);
            glVertex2f(x+1, y-1);
            glVertex2f(x-1, y-1);
            glVertex2f(x-1, y+1);
        }

        glColor3f(1, 0, 0);
        Player e = Game.obj.player;
        float x = e.x * 144 / Game.obj.fieldWidth + offx;
        float y = e.y * 144 / Game.obj.fieldHeight + offy;
        glVertex2f(x+1, y+1);
        glVertex2f(x+1, y-1);
        glVertex2f(x-1, y-1);
        glVertex2f(x-1, y+1);

        glEnd();
    }
}

class ScoreDraw : DrawObj {
    override void draw() {
        with (Game.obj()) {
            int offx1 = 500;
            int offx2 = 610;
            int offy = 190;
            drawString("SCORE", offx1, offy, 14, 18, 1,1,1);
            drawNumR(score, offx2, offy+20, 14, 18, 1,1,1);
            drawString("LIFE", offx1, offy+45, 14, 18, 1,1,1);
            drawNumR(playerLeft, offx2, offy+65, 14, 18, 1,1,1);
            drawString("TIME", offx1, offy+90, 14, 18, 1,1,1);
            if (mode != Mode.ENDLESS) {
                drawNumR(16000-turn, offx2, offy+110, 14, 18, 1,1,1);
            }
            else {
                drawNumR(turn, offx2, offy+110, 14, 18, 1,1,1);
            }
        }
    }
}

class PatternDraw : DrawObj {
    override void draw() {
        int i = 0;
        version (GNU) {
            int[int] m = Game.obj.patterns();
            foreach (int p; m.keys.sort) {
                int n = m[p];
                drawString("Pattern " ~ tostr(p+1) ~ " (" ~ tostr(n) ~ ")",
                           495, 340+i*22, 12, 16, 1,1,1);
                i++;
            }
        }
        else {
            foreach (int p, int n; Game.obj.patterns()) {
                drawString("Pattern " ~ tostr(p+1) ~ " (" ~ tostr(n) ~ ")",
                           495, 340+i*22, 12, 16, 1,1,1);
                i++;
            }
        }
    }
}
