private import charactor;
private import mygl;
private import texture;
private import game;
private import objspecs;
private import common;
private import sino;
private import rc;

class Particle : Charactor {
    mixin HasLife;
    mixin HasRelativePush;

    this(float x, float y, float sx, float sy) {
        this(x, y, sx, sy, 8);
    }

    this(float x, float y, float sx, float sy, float size) {
        static float[3] col = [ 1, 1, 1 ];
        this(x, y, sx, sy, size, col);
    }

    this(float x, float y, float sx, float sy, float size, float[3] col) {
        x_ = x;
        y_ = y;
        sx_ = sx;
        sy_ = sy;
        size_ = size;
        col_[] = col;
        life_ = 1;
    }

    override void move() {
        super.move();
        sx_ += rnd(2.0)-1.0;
        sy_ += rnd(2.0)-1.0;

        life_ -= 0.02;

        if (!Game.obj.isInside(x_, y_)) die();
    }

    override void draw() {
        if (!alive_) return;

        pushMatrix();

        GLfloat[] texcoord = Texture.prepare(TextureID.TEX_PARTICLE);

        glColor4f(col_[0], col_[1], col_[2], life_*alphaRatio);

        float tx = size_;
        float ty = size_;
        glBegin(GL_QUADS);
        glTexCoord2f(texcoord[0], texcoord[1]);
        glVertex2f(-tx, -ty);
        glTexCoord2f(texcoord[0], texcoord[3]);
        glVertex2f(-tx, ty);
        glTexCoord2f(texcoord[2], texcoord[3]);
        glVertex2f(tx, ty);
        glTexCoord2f(texcoord[2], texcoord[1]);
        glVertex2f(tx, -ty);
        glEnd();

        glDisable(GL_TEXTURE_2D);

        glPopMatrix();
    }

    float size_;
    float[3] col_;

    static void init() {
        alphaRatio = 0.01 * Sino.rc.getInt(`particleAlphaRatio`);
    }

private:
    static float alphaRatio;
}

class RevParticle : Particle {
    this(float x, float y, float sx, float sy, float size, float[3] col) {
        super(x, y, sx, sy, size, col);
        fx_ = x;
        fy_ = y;
        hx_.length = 50;
        hy_.length = 50;
    }

    override bool alive() {
        return turn_ < 180;
    }

    override void move() {
        if (turn_ < 49) {
            super.move();
            hx_[turn_] = x_-fx_;
            hy_[turn_] = y_-fy_;
        }
        else {
            turn_++;
            if (turn_ > 130) {
                life_ += 0.02;
                x_ = hx_[180-turn_] + Game.obj.player.x;
                y_ = hy_[180-turn_] + Game.obj.player.y;
            }
        }
    }

    override void draw() {
        if (!alive_) return;

        if (turn_ >= 50 && turn_ <= 130) return;

        super.draw();
    }

private:
    float fx_, fy_;
    float[] hx_, hy_;
}

