private import std.conv;
private import std.stream;
private import std.string;
private import log;
private import myio;

extern(C) void exit(int status);

class Rc {
public:
    this() {
    }

    void addDefault() {
    }
    void addResourceFile() {
        addResourceFile("data/sino.rc");
        addResourceFile("save/sino.rc");
    }
    void addResourceFile(char[] file) {
        auto File input;
        try {
            input = new File(file);
        }
        catch (Error e) {
            Log.error(e.toString());
        }

        while (!input.eof()) {
            char[] line = input.readLine();
            if (line.length == 0 || line[0] == '#') continue;
            char[][] tok = split(line);
            if (tok.length < 1) {
                Log.warn(file ~  ": resource file broken. (" ~ line ~ ")");
                continue;
            }

            if (tok.length == 1) {
                set(tok[0], ``);
                continue;
            }

            int valIndex = (tok[1] == "=") ? 2 : 1;

            set(tok[0], join(tok[valIndex..tok.length], " "));
        }
    }
    void addArgument(char args[][]) {
        for (int i = 1; i < args.length; i++) {
            if (args[i] == "-w") {
                set("timerEnable", "0");
            }
            else {
                if (args[i] != "-h") {
                    dout.writeLine(args[i] ~ ": invalid option");
                }
                dout.writeLine("usage: " ~ args[0] ~ " [options]");
                dout.writeLine("options:");
                dout.writeLine("  -w    \tnowait");
                dout.writeLine("  -m    \tnosound (yet)");
                dout.writeLine("  -l    \tlow resolution (yet)");
                dout.writeLine("  -b    \tno background (yet)");
                dout.writeLine("  -w    \twindow (yet)");
                dout.writeLine("  -h    \thelp");
                exit(1);
            } 
        }
    }

public:
    bool hasKey(char[] key) {
        return (key in rc_) !== null;
    }

    char[] get(char[] key) {
        if (!(key in rc_)) {
            Log.warn(key ~ " is not in resource");
            return "";
        }
        return rc_[key];
    }
    int getInt(char[] key) {
        return toInt(get(key));
    }
    bit getBit(char[] key) {
        return toInt(get(key)) != 0;
    }
    float getFloat(char[] key) {
        return atof(get(key));
    }

    void set(char[] key, char[] val) {
        rc_[key] = val;
    }

    char[][] keys() {
        return rc_.keys;
    }

	void dump(Stream s) {
		foreach (char[] k; rc_.keys) {
			s.writeLine(k ~ sep_ ~ rc_[k]);
		}
	}

	void dump(char[] filename) {
		auto File f = new File(filename, FileMode.OutNew);
		dump(f);
		f.close();
	}

    void dump() {
        dump(`save/sino.rc`);
    }

private:
    char[][char[]] rc_;
    static const char[] sep_ = " ";

};

private import std.stream;

